<?php

namespace App\Controllers\Admin;

use Hermawan\DataTables\DataTable;
use App\Controllers\BaseController;

class ManageProduct extends BaseController
{
    protected $uModel;
    protected $pModel;
    protected $vModel;

    public function __construct()
    {
        $this->uModel = new \App\Models\UserModel();
        $this->pModel = new \App\Models\ProductModel();
        $this->vModel = new \App\Models\VariantModel();
    }

    public function index()
    {
        // DB Pusher
        update_database();

        $data = [
            'title' => 'List Product',
            'user' => $this->uModel->get_user(['username' => session()->get('username')]),
        ];
        return view('Admin/product', $data);
    }


    public function productListApi()
    {
        $builder = $this->pModel->select('products.id as product_id, products.name, products.price, products.price_discount, products.category, products.status, products.badge, products.slug, COUNT(product_variants.id) as variant_count')
            ->join('product_variants', 'product_variants.product_id = products.id', 'left')
            ->groupBy('products.id');

        return DataTable::of($builder)
            ->setSearchableColumns(['product_id', 'products.name', 'products.price', 'products.price_discount', 'products.category', 'products.status'])
            ->format('name', function ($value) {
                return ucwords($value);
            })
            ->format('price', function ($value) {
                return rupiah($value);
            })
            ->format('price_discount', function ($value) {
                return rupiah($value);
            })
            ->edit('status', function ($row) {
                if ($row->status == 1) {
                    return '<span class="badge bg-success">Tersedia<span>';
                }
                return '<span class="badge bg-danger">Habis<span>';
            })
            ->add('action', function ($row) {
                $urlAction = site_url('admin/product/edit/' . $row->product_id);
                $edit = '<a href="' . $urlAction . '" class="btn btn-primary btn-sm">EDIT</a>';
                $delete = '<button type="button" class="btn btn-danger btn-sm product-delete" data-product_id="' . $row->product_id . '">DELETE</button>';
                return "$edit $delete";
            })
            ->toJson(true);
    }

    public function create()
    {
        if ($this->request->getPost()) {
            return $this->save();
        }

        // DB Pusher
        update_database();

        $data = [
            'title' => 'Add Product',
            'product' => [],
            'variant' => []
        ];

        return view('Admin/manage_product', $data);
    }

    private function save()
    {
        $form_rules = [
            'name' => [
                'label' => 'Nama Produk',
                'rules' => 'required|min_length[3]|max_length[255]|is_unique[products.name]',
                'errors' => [
                    'required' => '{field} tidak boleh kosong',
                    'min_length' => '{field} minimal 3 karakter',
                    'max_length' => '{field} maksimal 255 karakter',
                    'is_unique' => '{field} sudah ada, gunakan nama lain',
                ]
            ],
            'price' => 'required|numeric',
            'price_discount' => 'required|numeric',
            'category' => 'required',
            'status' => 'in_list[1,0]',
            'badge' => 'in_list[terlaris,mantap,fs,diskon]|permit_empty',
            'image' => 'uploaded[image]|max_size[image,1024]|is_image[image]|mime_in[image,image/jpg,image/jpeg,image/png,image/svg,image/webp,image/gif]',
        ];

        if (!$this->validate($form_rules)) {
            return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
        }

        $name = (string) $this->request->getPost('name');
        $slug = url_title($name, '-', true);
        $send = (string) $this->request->getPost('time_send');
        $width = (string) $this->request->getPost('image_width');
        $category = (string) $this->request->getPost('category');

        // check if slug exist then add random string to slug
        if ($this->pModel->getProduct(['slug' => $slug])) {
            $slug = $slug . '-' . random_string('alnum', 5);
        }

        // handle image upload
        $image = $this->request->getFile('image');
        if ($image->isValid()) {
            $newName = $image->getRandomName();
            $image->move('assets/images/product', $newName);
            $image = $newName;
        } else {
            $image = 'default.png';
        }

        $data = [
            'name' => ucwords($name),
            'hide_price' => request()->getPost('hide_price') ?: 0,
            'price' => request()->getPost('price'),
            'price_discount' => request()->getPost('price_discount'),
            'category' => ucwords($category),
            'status' => request()->getPost('status'),
            'badge' => request()->getPost('badge'),
            'time_send' => preg_match("/\d+/", $send, $matches) ? $matches[0] : null,
            'image' => $image,
            'image_width' => $width ? $width . "%" : null,
            'image_color' => request()->getPost('image_color'),
            'description' => request()->getPost('description'),
            'other_field' => ucwords((string) request()->getPost('other_field')),
            'must_field' => request()->getPost('must_field') ? 1 : 0,
            'type_field' => request()->getPost('type_field') ?: 'input',
            'default_field' => request()->getPost('default_field'),
            'slug' => $slug,
        ];

        $product_id = $this->pModel->insert($data, true);
        $variants = $this->request->getPost('variants');

        if ($variants) {
            $data_variant = [];
            foreach ($variants as $variant) {
                if (!$variant['harga']) {
                    continue;
                }

                $slug = $variant['nama'] ? url_title($variant['nama'] . " " . random_string('alnum', 6)) : random_string('alnum', 15);
                $data_variant[] = [
                    'id' => isset($variant['id']) ? $variant['id'] : null,
                    'product_id' => $product_id,
                    'slug' => $slug,
                    'name' => ucwords($variant['nama']) ?: $data['name'],
                    'info' => ucwords($variant['info']),
                    'price' => $variant['harga'] ?: $data['price_discount'],
                    'status' => $variant['status'],
                ];
            }

            if ($data_variant) {
                $this->vModel->insertBatch($data_variant);
            } else {
                return redirect()->to('admin/product/edit/' . $product_id)->withInput()->with('errors', ['variants' => 'Variants tidak boleh kosong']);
            }
        }

        // ambil data produk
        $product = $this->pModel->getProduct(['id' => $product_id]);
        // generate OG images
        $og_theme = getStore('og_theme');
        generateOgImage($og_theme, $product);

        return redirect()->to('admin')->with('success', 'Product berhasil ditambahkan');
    }

    public function edit($id)
    {
        if ($this->request->getPost()) {
            return $this->update($id);
        }

        // DB Pusher
        update_database();

        $product = $this->pModel->getProduct(['id' => $id]);
        if (!$product) {
            return redirect()->to('admin/product')->with('error', 'Product tidak ditemukan');
        }

        $images = product_images($id);

        $data = [
            'title' => 'Edit Product',
            'product' => $product,
            'variant' => $this->vModel->getVariants(['product_id' => $id]),
            'images' => $images ?: null,
        ];

        return view('Admin/manage_product', $data);
    }

    private function update($id)
    {
        $form_rules = [
            'name' => 'required',
            'price' => 'required|numeric',
            'price_discount' => 'required|numeric',
            'category' => 'required',
            'status' => 'in_list[1,0]',
            'badge' => 'in_list[terlaris,mantap,fs,diskon]|permit_empty',
            'image' => 'max_size[image,1024]|is_image[image]|mime_in[image,image/jpg,image/jpeg,image/png]',
        ];

        if (!$this->validate($form_rules)) {
            return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
        }

        $product = $this->pModel->getProduct(['id' => $id]);

        $name = (string) $this->request->getPost('name');
        $send = (string) $this->request->getPost('time_send');
        $info = (string) $this->request->getPost('info');
        $category = (string) $this->request->getPost('category');
        $width = (string) $this->request->getPost('image_width');
        $old_image = $product['image'];

        // handle image upload
        $image = $this->request->getFile('image');
        if ($image->isValid()) {
            $newName = $image->getRandomName();
            $image->move('assets/images/product', $newName);
            $image = $newName;

            if ($old_image != 'default.png') {
                $img_old = 'assets/images/product/' . $old_image;
                if (file_exists($img_old)) {
                    unlink($img_old);
                }
            }
        } else {
            $image = $old_image;
        }

        $data = [
            'name' => ucwords($name),
            'hide_price' => $this->request->getPost('hide_price') ?: 0,
            'price' => $this->request->getPost('price'),
            'price_discount' => $this->request->getPost('price_discount'),
            'qty' => $this->request->getPost('qty'),
            'category' => ucwords($category),
            'status' => $this->request->getPost('status'),
            'badge' => $this->request->getPost('badge'),
            'time_send' => preg_match("/\d+/", $send, $matches) ? $matches[0] : null,
            'image' => $image,
            'image_width' => $width ? $width . "%" : null,
            'image_color' => $this->request->getPost('image_color'),
            'info' => ucwords($info),
            'description' => $this->request->getPost('description'),
            'other_field' => ucwords((string) request()->getPost('other_field')),
            'must_field' => request()->getPost('must_field') ? 1 : 0,
            'type_field' => request()->getPost('type_field') ?: 'input',
            'default_field' => request()->getPost('default_field'),
        ];

        $this->pModel->update($id, $data);
        $variants = $this->request->getPost('variants');

        if ($variants) {
            $data_variant = [];
            foreach ($variants as $variant) {
                if (!$variant['nama'] && !$variant['harga'] && !$variant['info']) {
                    continue;
                }

                preg_match("/[a-zA-Z0-9]+/", $variant['nama'], $for_slugs);
                $slug =  isset($variant['slug']) ? $variant['slug'] : url_title($for_slugs[0] . " " . random_string('alnum', 6), '-', true);
                $data_variant[] = [
                    'id' => isset($variant['id']) ? $variant['id'] : null,
                    'product_id' => $product['id'],
                    'slug' => $slug,
                    'name' => trim(ucwords($variant['nama'])),
                    'info' => trim(ucwords($variant['info'])),
                    'price' => $variant['harga'],
                    'status' => $variant['status'],
                ];
            }

            if ($data_variant) {
                $this->vModel->upsert($data_variant);
            } else {
                return redirect()->back()->withInput()->with('failed', 'Variants tidak boleh kosong');
            }
        }
        // sort product image index
        $images = $this->request->getPost('image_order');
        // dd($images);
        if ($images) {
            $db = \Config\Database::connect();
            $builder = $db->table('product_images');
            foreach ($images as $key => $image) {
                $builder->where('id', $image);
                $builder->update(['sort_number' => $key + 1]);
            }
        }

        // ambil data produk terbaru
        $product = $this->pModel->getProduct(['id' => $id]);
        // generate OG images
        $og_theme = getStore('og_theme');
        generateOgImage($og_theme, $product);

        // return response
        return redirect()->back()->with('success', 'Product berhasil diupdate');
    }

    public function delete()
    {
        $id = $this->request->getPost('product_id');
        if (!$id) {
            return $this->response->setStatusCode(400)->setBody('ID tidak ditemukan');
        }

        $product = $this->pModel->getProduct(['id' => $id]);
        if (!$product) {
            return $this->response->setStatusCode(404)->setBody('Product tidak ditemukan');
        }

        // delete variants
        $variants = $this->vModel->getVariants(['product_id' => $id]);
        if ($variants) {
            foreach ($variants as $variant) {
                $this->vModel->delete($variant['id']);
            }
        }

        // delete image if not default.png
        if ($product['image'] != 'default.png') {
            unlink('assets/images/product/' . $product['image']);
        }
        // delete og image if exists
        $og_image = 'assets/images/og/product/' . $product['slug'] . '.jpg';
        if (file_exists($og_image)) {
            unlink($og_image);
        }

        // delete product
        $this->pModel->delete($id);
        return $this->response->setStatusCode(200)->setBody('Product berhasil dihapus');
    }

    public function delete_variant()
    {
        $id = $this->request->getPost('variant_id');
        if (!$id) {
            return $this->response->setStatusCode(400)->setBody('ID tidak ditemukan');
        }

        $variant = $this->vModel->getVariant(['id' => $id]);
        if (!$variant) {
            return $this->response->setStatusCode(404)->setBody('Variant tidak ditemukan');
        }

        $this->vModel->delete($id);
        return $this->response->setStatusCode(200)->setBody('Variant berhasil dihapus');
    }

    public function upload_image()
    {
        $product_id = $this->request->getPost('product_id');
        $images = $this->request->getFiles();
        if (!$images) {
            return $this->response->setStatusCode(400)->setBody('Gambar tidak ditemukan');
        }

        $db = \Config\Database::connect();
        $builder = $db->table('product_images');

        $data = [];
        foreach ($images['images'] as $image) {
            if ($image->isValid()) {
                $newName = $image->getRandomName();
                $image->move('assets/images/product/extra', $newName);
                $data[] = $newName;
            }
        }

        // save to database dan sort_number berdasarkan index array
        if ($data) {
            $lastSortNumber = $builder->selectMax('sort_number')->getWhere(['product_id' => $product_id])->getRowArray();
            $lastSortNumber = $lastSortNumber['sort_number'] ?: 0;
            $dataInsert = [];
            foreach ($data as $key => $value) {
                $dataInsert[] = [
                    'product_id' => $product_id,
                    'sort_number' => $lastSortNumber + $key + 1,
                    'image_url' => $value,
                ];
            }
            $builder->insertBatch($dataInsert);
        }

        // get list image
        $builder->select('id, sort_number, image_url');
        $builder->where('product_id', $product_id);
        $data_images = $builder->get()->getResultArray();
        // return response
        $response = [
            'status' => 200,
            'success' => true,
            'message' => 'Gambar berhasil diupload',
            'data' => $data_images,
        ];
        return $this->response->setStatusCode(200)->setJSON($response);
    }

    public function delete_image()
    {
        $id = $this->request->getPost('id');
        if (!$id) {
            return $this->response->setStatusCode(400)->setJSON([
                'status' => false,
                'message' => 'ID tidak ditemukan'
            ]);
        }

        $db = \Config\Database::connect();
        $builder = $db->table('product_images');
        $image = $builder->getWhere(['id' => $id])->getFirstRow('array');
        if (!$image) {
            return $this->response->setStatusCode(404)->setJSON([
                'status' => false,
                'messsage' => 'Gambar tidak ditemukan'
            ]);
        }

        // delete on db
        $builder->where('id', $id);
        $builder->delete();
        $image_url = $image['image_url'];
        // check if image exist then delete
        $path = 'assets/images/product/extra/' . $image_url;
        if (file_exists($path)) {
            unlink($path);
        }
        return $this->response->setStatusCode(200)->setJSON([
            'status' => true,
            'message' => 'Gambar berhasil dihapus'
        ]);
    }

    public function sort()
    {
        if (request()->getPost('product_id')) return $this->sort_product();

        $products = $this->pModel->getProductSorted();
        $data = [
            'title' => 'Sort Product',
            'products' => $products,
        ];

        return view('Admin/product_sort', $data);
    }

    private function sort_product()
    {
        $product_id = $this->request->getPost('product_id');
        if (!$product_id) {
            return $this->response->setStatusCode(400)->setBody('ID harus diisi');
        }

        $data = [];
        foreach ($product_id as $key => $value) {
            $data[] = [
                'id' => $value,
                'sort_number' => $key + 1,
            ];
        }

        // save to database dan sort_number berdasarkan index array
        if ($data) {
            $this->pModel->updateBatch($data, 'id');
        }

        // return response
        return $this->response->setStatusCode(200)->setBody('Product berhasil diurutkan');
    }
}
