<?php

namespace App\Controllers\Admin;

use App\Controllers\BaseController;
use Hermawan\DataTables\DataTable;

class Posts extends BaseController
{
    protected $postModel;

    public function __construct()
    {
        $this->postModel = new \App\Models\PostModel();
    }

    public function index()
    {
        $data = [
            'title' => 'Posts',
            'posts' => $this->postModel->findAll(),
        ];
        return view('Admin/posts/index', $data);
    }

    public function create_slug($title)
    {
        $slug = substr(url_title($title, '-', true), 0, 15);
        $slug .= "-" . random_string('alnum', 6);
        return $slug;
    }

    public function postListApi()
    {
        $builder = $this->postModel->select('id, image, title, slug, status, created_at');

        return DataTable::of($builder)
            ->setSearchableColumns(['title', 'status', 'created_at'])
            ->format('image', function ($value) {
                return '<img src="' . base_url('assets/images/post/' . $value) . '" alt="' . $value . '" class="img-fluid" style="max-height: 50px;">';
            })
            ->edit('title', function ($row) {
                $title = strlen($row->title) > 30 ? substr($row->title, 0, 30) . '...' : $row->title;
                return '<a class="link-dark" href="' . site_url('news/' . $row->slug) . '" target="_new">' . $title . '</a>';
            })
            ->format('created_at', function ($value) {
                return date('d/m/Y - H:i:s', strtotime($value));
            })
            ->add('action', function ($data) {
                return '<a href="' . base_url('admin/posts/edit/' . $data->id) . '" class="btn btn-sm btn-warning">Edit</a>
                <button type="button" class="btn btn-sm btn-danger postDelete" data-id_post="' . $data->id . '">Delete</button>';
            })
            ->edit('status', function ($row) {
                if ($row->status == 1) {
                    return '<span class="badge bg-success">Published<span>';
                }
                return '<span class="badge bg-danger">Draft<span>';
            })
            ->toJson(true);
    }

    public function delete()
    {
        if ($this->request->isAJAX()) {
            $id_post = $this->request->getPost('id_post');
            $this->postModel->delete($id_post);
            return json_encode(['status' => 'success']);
        }
    }

    public function create()
    {
        if (request()->getPost()) {
            return $this->store();
        }

        $data = [
            'title' => 'Add Post',
        ];
        return view('Admin/posts/create', $data);
    }

    private function store()
    {
        $rules = [
            'title' => 'required',
            'content' => 'required',
            'image' => 'uploaded[image]|max_size[image,2048]|is_image[image]',
            'status' => 'required|in_list[0,1]',
        ];

        if (!$this->validate($rules)) {
            $validation = \Config\Services::validation();
            return redirect()->back()->withInput()->with('errors', [$validation->getErrors()]);
        }

        $image = $this->request->getFile('image');
        if ($image->isValid()) {
            $newName = $image->getRandomName();
            $image->move('assets/images/post', $newName);
        }

        $title = (string) $this->request->getPost('title');

        $data = [
            'title' => ucwords($title),
            'slug' => $this->create_slug($title),
            'content' => $this->request->getPost('content'),
            'image' => isset($newName) ? $newName : 'default.jpg',
            'status' => $this->request->getPost('status'),
        ];

        $this->postModel->insert($data);
        return redirect()->to('admin/posts')->with('success', 'Berhasil membuat postingan baru');
    }

    public function edit($id)
    {
        if (request()->getPost()) {
            return $this->update($id);
        }

        $data = [
            'title' => 'Edit Post',
            'post' => $this->postModel->find($id),
        ];
        return view('Admin/posts/edit', $data);
    }

    private function update($id)
    {
        $rules = [
            'title' => 'required',
            'content' => 'required',
            'image' => 'permit_empty|max_size[image,2048]|is_image[image]',
            'status' => 'required|in_list[0,1]',
        ];

        if (!$this->validate($rules)) {
            $validation = \Config\Services::validation();
            return redirect()->back()->withInput()->with('errors', [$validation->getErrors()]);
        }

        $post = $this->postModel->find($id);

        $image = $this->request->getFile('image');

        if ($image->isValid()) {
            $newName = $image->getRandomName();
            $image->move('assets/images/post', $newName);
            // remove old image
            if ($post['image'] != 'default.jpg') {
                unlink('assets/images/post/' . $post['image']);
            }
        }

        $title = (string) $this->request->getPost('title');

        $data = [
            'title' => ucwords($title),
            'slug' => $this->create_slug($title),
            'content' => $this->request->getPost('content'),
            'status' => $this->request->getPost('status'),
        ];

        if (isset($newName)) {
            $data['image'] = $newName;
        }

        $this->postModel->update($id, $data);
        return redirect()->to('admin/posts')->with('success', 'Berhasil memperbarui postingan');
    }
}
